@@ -1,5 +1,5 @@
 /*-*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: Library.java,v 1.8 2000/08/30 17:14:32 metlov Exp $
+ * $Id: Library.java,v 1.9 2000/09/09 19:51:48 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -384,7 +384,7 @@ public class Library {
     return most_specific;
   };
 
-  private static String describe(String name,Class[] params) {
+  protected static String describe(String name,Class[] params) {
     String res="";
     StringBuffer invp=new StringBuffer();
     invp.append(name); 
