@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
- * $Id: OPcall.java,v 1.3 2000/08/28 20:05:53 metlov Exp $
+ * $Id: OPcall.java,v 1.4 2000/09/09 19:51:48 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -21,8 +21,9 @@ import java.util.Stack;
 
 public class OPcall extends OPfunction {
   private Member m; // member to eval (null for the local variable access)
-  private Class[] reqParamTypes;
-  private int lvarn=0; // local variable number (in case m=null)
+  // local variable number (in case m=null), number of formal parameters of
+  // the method to call otherwise.
+  private int nplv=0;
   private boolean attemptEval=true;
 
   /**
@@ -41,16 +42,23 @@ public class OPcall extends OPfunction {
     throws CompilationException {
     this.m=m;
     this.attemptEval=attemptEval;
-    reqParamTypes=m.getParameterTypes();
+    Class[] reqParamTypes=m.getParameterTypes();
+    nplv=reqParamTypes.length;
+    
+    int thisIdx=0;
+    if ((m.getModifiers() & 0x0008) == 0) { // method is not static
+      thisIdx=-1;
+      nplv++;
+    };
 
-    int thisIdx=((m.getModifiers() & 0x0008) > 0?0:-1);
     // convert formal and actual paramter types and "this", if needed    
     for(int i=reqParamTypes.length-1;i>=thisIdx;i--) {
       Class cReq=(i>=0?reqParamTypes[i]:m.getDeclaringClass());
       
       // add convert type OP
-      if (typesStk.peek()!=cReq)
-        list.addAfter((OP)paramOPs.peek(),new OPunary(typesStk,cReq,i<0));
+      if ((typesStk.peekID()==10) || (typesStk.peek()!=cReq))
+        list.addAfter((OP)paramOPs.peek(),
+                      new OPunary(typesStk,TypesStack.typeID(cReq),cReq,i<0));
       
       // throw out
       paramOPs.pop();
@@ -72,8 +80,7 @@ public class OPcall extends OPfunction {
    */
   public OPcall(TypesStack typesStk, int lvarn, Class type) {
     this.m=null;
-    this.reqParamTypes=new Class[0];
-    this.lvarn=lvarn;
+    this.nplv=lvarn;
     typesStk.push(type);
     resID=typesStk.peekID();
     resType=type;
@@ -83,9 +90,8 @@ public class OPcall extends OPfunction {
    * Returns number of parameters for this function.
    */
   public int getNParams() {
-    int nparams=reqParamTypes.length;
-    if ((m!=null) && ((m.getModifiers() & 0x0008)==0)) nparams++;
-    return nparams;
+    if (m==null) return 0;
+    return nplv;
   };
 
   /**
@@ -97,10 +103,11 @@ public class OPcall extends OPfunction {
     if (!attemptEval) return; // do not eval if not requested.
     if (m==null) return; // can't eval local variable loads
     if ((m.getModifiers() & 0x0008) ==0) return; // can't eval non-static
-    if ((resID==8) && !(TypesStack.isString(resType))) return;
+    if ((resID>7) && (resID!=11)) return; // can't hold object constants
+
     try {
       Object[] params=null;
-      int nParams=reqParamTypes.length;
+      int nParams=nplv;
       if (nParams>0) {
         params=new Object[nParams];
         OPload cOPload= (OPload) prev;
@@ -140,10 +147,10 @@ public class OPcall extends OPfunction {
   // compilation code
 
   protected void compile_pre(ClassFile cf) {
-    if ((m!=null) && (reqParamTypes.length>0)) cf.labels_block();
+    if (m!=null) cf.labels_block();
   };
 
-  protected void compile_par(ClassFile cf) {
+  protected void compile_par(ClassFile cf, int n) {
     cf.ensure_value();
     // if ((m!=null) && (reqParamTypes.length>0)) cf.labels_unblock();
   };
@@ -162,17 +169,13 @@ public class OPcall extends OPfunction {
   //    {0x19c4,0x19,0x2a,0x2b,0x2c,0x2d}  // L
   //  };
 
-  // f - field, m - method, c - constructor, sX -- static X
-  //               f    sf   m    sm   c    sc
-  private int[] callinsn={0xb4,0xb2,0xb6,0xb8,0xb7,0xb7};
-
   protected void compile(ClassFile cf) {
     if (m==null) {
       // load the local variable with a given number
       int lvt=resID-4;
       if (lvt<0) lvt=0;
 
-      int lvarn_translated=cf.paramsVars[lvarn];
+      int lvarn_translated=cf.paramsVars[nplv];
 
       if (lvarn_translated<4) 
         cf.code(0x1a+lvt*4+lvarn_translated);
@@ -183,16 +186,12 @@ public class OPcall extends OPfunction {
         cf.codeI(lvarn_translated);
       };
     } else { 
-      if (reqParamTypes.length>0) cf.labels_unblock();
-
+      cf.labels_unblock();
+      
       // call the method / get field
-      int mdfrs=m.getModifiers();
-      int opcodeindex=(((mdfrs >>> 16)-1) << 1) + ((mdfrs & 0x0008) >>> 3);
-      cf.code(callinsn[opcodeindex]);
-      cf.codeI(cf.getIndex(m,10));
-
-      int nParams=reqParamTypes.length;
-      if ((mdfrs & 0x0008) == 0) nParams++; // non-static has one more param
+      m.code(cf);
+      
+      int nParams=nplv;
       while(nParams-->0) cf.typesStk.pop();
     };
     
@@ -201,7 +200,7 @@ public class OPcall extends OPfunction {
 
   public String toString() {
     if (m==null)
-      return "{"+lvarn+"}";
+      return "{"+nplv+"}";
     else
       return m.getName();
   };
