@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
- * $Id: OPfunction.java,v 1.1 2000/02/11 22:08:18 metlov Exp $
+ * $Id: OPfunction.java,v 1.2 2000/09/09 19:51:48 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -40,7 +40,8 @@ public abstract class OPfunction extends OP {
   /**
    * Called to generate the code after a parameter is processed.
    * @param cf class file to write the code into.
+   * @param n the number of parameter (1 is first).
    */
-  protected void compile_par(ClassFile cf) {
+  protected void compile_par(ClassFile cf, int n) {
   };
 };
