@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: TestSuite.java,v 1.20 2000/08/30 17:14:32 metlov Exp $
+ * $Id: TestSuite.java,v 1.23 2000/09/10 15:59:34 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -57,8 +57,16 @@ public class TestSuite {
     testError("",null,lib,0,o);
     testError(" ",null,lib,1,o);
     testError("tru",null,lib,1,o);
+
+    // two following tests may fail if the parser bug is not fixed
+    // this fixing is done in the release script but the development
+    // version always fails these tests
+    boolean old_abort_at_error=abort_at_error;
+    abort_at_error=false;
     testError("1=",null,lib,2,o);
     testError("1=2",null,lib,3,o);
+    abort_at_error=old_abort_at_error;
+
     testError("1.0-+1.0",null,lib,5,o);
     testError("1.0&1.0",null,lib,4,o);
     testError("-",null,lib,1,o);
@@ -289,7 +297,10 @@ public class TestSuite {
     testExpression("5*arr[1]-66",new Double(25-66),Double.TYPE,rtp,lib,o);
     testExpression("5*(arrs[1]+arrs[2])-66",new Double(25-66),Double.TYPE,rtp,lib,o);
     testExpression("5*(arrs[1]+arrs[(int)round(arr[0]-arrs[2])+1])-66",new Double(25-66),Double.TYPE,rtp,lib,o);
-    
+
+    testExpression("arrDoubleJELObj1[0]+arrDoubleJELObj1[1]",
+                   new Double(3.0), null, rtp, lib,o);
+
     o.println(); o.println("----- TESTING COMPILED-in EXCEPTIONS (x=5)------");
     o.println();
     
@@ -298,10 +309,11 @@ public class TestSuite {
 
     o.println(); o.println("----- DOT OPERATOR ------");
     // Construct a new library
-    Class[] dotLib=new Class[2];
+    Class[] dotLib=new Class[3];
     try {
       dotLib[0]=Class.forName("java.lang.String");
       dotLib[1]=Class.forName("java.lang.Double");
+      dotLib[2]=Class.forName("gnu.jel.reflect.Double");
     } catch(ClassNotFoundException e) {
       System.out.println(e);
       System.exit(1);
@@ -328,6 +340,11 @@ public class TestSuite {
     testExpression("arr[byteObj]", new Double(6.0), null, rtp, lib,o);
     testExpression("byteObj+1.0", new Double(3.0), null, rtp, lib,o);
 
+    testExpression("arrDoubleJELObj1[0].getValue()+"+
+                   "arrDoubleJELObj1[1].getValue()",
+                   new Double(3.0), null, rtp, lib,o);
+
+
     o.println(); o.println("----- DYNAMIC VARIABLES ------");
     lib=new Library(staticLib,dynamicLib,dotLib,vp);
     vp.addProperty("p1","p1value");
@@ -335,18 +352,287 @@ public class TestSuite {
     vp.addProperty("p1.s2","p1s2value");
     vp.addProperty("p1.d1",VariableProvider.makeJELDoubleObject(1.0));
     vp.addProperty("p1.s2.ss1","p1s2ss1value");
+    vp.addProperty("p1.b1t",VariableProvider.makeJELBooleanObject(true));
+    vp.addProperty("p1.b1f",VariableProvider.makeJELBooleanObject(false));
     testExpression("p1", "p1value", null, rtp, lib,o);
     testExpression("p1.s1", "p1s1value", null, rtp, lib,o);
     testExpression("p1.s2", "p1s2value", null, rtp, lib,o);
     testExpression("p1.s2.ss1","p1s2ss1value", null, rtp, lib,o);
     testExpression("p1.s2.length()", new Integer(9), null, rtp, lib,o);
     testExpression("p1+(p1.d1+p1.s2.length()+1)", "p1value11.0", null, rtp, lib,o);
+    testExpression("round(p1.d1)", new Long(1), null, rtp, lib,o);
+    testExpression("p1.d1>0", new Boolean(true), null, rtp, lib,o);
+    testExpression("p1.d1>0?p1.d1:3.0", new Double(1.0), null, rtp, lib,o);
+    testExpression("p1.b1t", new Boolean(true), null, rtp, lib,o);
+    testExpression("(boolean)p1.b1t", new Boolean(true), null, rtp, lib,o);
+    testExpression("p1.b1t?1:0", new Byte((byte)1), null, rtp, lib,o);
+
+    testExpression("aarr[1][0]", new Double(3.0), null, rtp, lib,o);
+
+    testExpression("aarrDouble[1][0].doubleValue()", 
+                   new Double(3.0), null, rtp, lib,o);
+
+    testExpression("aarrDouble[1][0]", 
+                   new Double(3.0), null, rtp, lib,o);
+
+    testExpression("\"\"+aarr[1][0]","3.0", null, rtp, lib,o);
+    testExpression("\"\"+aarrDouble[1][0]","3.0", null, rtp, lib,o);
+    testExpression("\"\"+aarrDouble[1][0].doubleValue",
+                   "3.0", null, rtp, lib,o);
+
+    testExpression("\"\"+\"\"+aarr[1][0]","3.0", null, rtp, lib,o);
+    testExpression("\"a\"+\"b\"+aarr[1][0]+\"c\"","ab3.0c", null, rtp, lib,o);
+
+
+    for(int k=0;k<2;k++) {
+      o.println(); 
+      o.println("----- THOROUGNLY TEST UNARY OPERATIONS  ------");
+
+      String[][] prefixes={{"val","val","val"},{"arr","arr","arr"}};
+      String[][] suffixes={{"","Obj","JELObj"},{"[0]","Obj[0]","JELObj[0]"}};
+      
+      testUnaryPrimitive(0,6,lib,rtp,-1,o,prefixes[k],suffixes[k]); // -
+      testUnaryPrimitive(1,5,lib,rtp,0xFFFFFFFFFFFFFFFEL,o,
+                         prefixes[k],suffixes[k]); // ~
+      testUnaryPrimitive(2,1,lib,rtp,0,o,prefixes[k],suffixes[k]); // !
+
+      o.println(); 
+      o.println("----- THOROUGNLY TEST BINARY OPERATIONS  ------");
+
+
+      testBinaryPrimitive(0,45,lib,rtp,2,o,prefixes[k],suffixes[k]); // +
+      testBinaryPrimitive(1,45,lib,rtp,0,o,prefixes[k],suffixes[k]); // -
+      testBinaryPrimitive(2,45,lib,rtp,1,o,prefixes[k],suffixes[k]); // *
+      testBinaryPrimitive(3,45,lib,rtp,1,o,prefixes[k],suffixes[k]); // /
+      testBinaryPrimitive(4,45,lib,rtp,0,o,prefixes[k],suffixes[k]); // %
+      testBinaryPrimitive(5,26,lib,rtp,1,o,prefixes[k],suffixes[k]); // &
+      testBinaryPrimitive(6,26,lib,rtp,1,o,prefixes[k],suffixes[k]); // |
+      testBinaryPrimitive(7,26,lib,rtp,0,o,prefixes[k],suffixes[k]); // ^
+      testBinaryPrimitive(8,46,lib,rtp,1,o,prefixes[k],suffixes[k]); // ==
+      testBinaryPrimitive(9,46,lib,rtp,0,o,prefixes[k],suffixes[k]); // !=
+      testBinaryPrimitive(10,45,lib,rtp,0,o,prefixes[k],suffixes[k]); // <
+      testBinaryPrimitive(11,45,lib,rtp,1,o,prefixes[k],suffixes[k]); // >=
+      testBinaryPrimitive(12,45,lib,rtp,0,o,prefixes[k],suffixes[k]); // >
+      testBinaryPrimitive(13,45,lib,rtp,1,o,prefixes[k],suffixes[k]); // <=
+      testBinaryPrimitive(14,25,lib,rtp,2,o,prefixes[k],suffixes[k]); // <<
+      testBinaryPrimitive(15,25,lib,rtp,0,o,prefixes[k],suffixes[k]); // >>
+      testBinaryPrimitive(16,25,lib,rtp,0,o,prefixes[k],suffixes[k]); // >>>
+      testBinaryPrimitive(17,1,lib,rtp,1,o,prefixes[k],suffixes[k]); // &&
+      testBinaryPrimitive(18,1,lib,rtp,1,o,prefixes[k],suffixes[k]); // ||
+
+    };
+
 
     o.println(); o.println("^^^^^^^^^^^^ SCORE success/failure = "+
                            successes+"/"+failures+
                            " ^^^^^^^^^^^^" );
   };
 
+
+  // Tests a given binary operation on all primitive types
+  private static void testUnaryPrimitive(int code,int npbc,
+                                         Library lib, Object[] context,
+                                         long resVal, PrintStream o,
+                                         String[] prefixes,
+                                         String[] suffixes) {
+    String[] typeNames={"Boolean","Byte","Character","Short","Integer",
+                        "Long","Float","Double"};
+
+    String[] opSymbols={"-","~","!"};
+    
+    Object[] typeConstants={
+      new java.lang.Boolean(true),
+      new java.lang.Byte((byte)1),
+      new java.lang.Character((char)1),
+      new java.lang.Short((short)1),
+      new java.lang.Integer(1),
+      new java.lang.Long(1),
+      new java.lang.Float((float)1.0),
+      new java.lang.Double(1.0)};
+
+    int npbcActual=0;
+
+    for(int i=0;i<8;i++) {
+        // determine the result type independently
+        int resID=-1;
+        try {
+          OPlist list=new OPlist();
+          TypesStack typeStk=new TypesStack();
+          OP o1=new OPload(typeStk,typeConstants[i]);
+          list.addLast(o1);
+
+          list.addLast(new OPunary(typeStk,code));
+          resID=typeStk.peekID();
+          npbcActual++;
+        } catch (CompilationException exc) {
+        };
+        Object res=null;
+        if (resID>=0) { // result exists test it
+          // construct the resulting object
+          switch (resID) {
+          case 0:
+            res=new java.lang.Boolean(resVal>0?true:false);
+            break;
+          case 1:
+            res=new java.lang.Byte((byte)resVal);
+            break;
+          case 2:
+            res=new java.lang.Character((char)resVal);
+            break;
+          case 3:
+            res=new java.lang.Short((short)resVal);
+            break;
+          case 4:
+            res=new java.lang.Integer((int)resVal);
+            break;
+          case 5:
+            res=new java.lang.Long(resVal);
+            break;
+          case 6:
+            res=new java.lang.Float((float)resVal);
+            break;
+          case 7:
+            res=new java.lang.Double((double)resVal);
+            break;
+          default:
+            o.print("*** : ");            
+            o.println("The result of unary operation \""+opSymbols[code]+
+                      "\" on "+TypesStack.specialTypes[i]+
+                      " is not primitive type ID="+resID);
+            FAIL(o);
+          };
+        };
+
+        for(int k=0;k<prefixes.length;k++) {
+          for(int m=k;m<prefixes.length;m++) {
+            String op1=prefixes[k]+typeNames[i]+suffixes[k];
+            String expr=opSymbols[code]+op1;
+            if (res!=null)
+              testExpression(expr,res,null,context,lib,o);
+            else
+              testError(expr,null,lib,1,o);   
+          };
+        };
+        
+      };
+    
+    o.print("*=*=*= : the total number of successful operations "+npbcActual);
+    if (npbcActual==npbc) {
+      o.println(".");
+      OK(o);
+    } else {
+      o.println(" must be "+npbc);
+      FAIL(o);
+    };
+  };
+
+  
+  // Tests a given binary operation on all primitive types
+  private static void testBinaryPrimitive(int code,int npbc,
+                                          Library lib, Object[] context,
+                                          int resVal, PrintStream o,
+                                         String[] prefixes,
+                                         String[] suffixes) {
+    String[] typeNames={"Boolean","Byte","Character","Short","Integer",
+                        "Long","Float","Double"};
+
+    String[] opSymbols=(String[])TableKeeper.getTable("binOpSymbols");
+    
+    Object[] typeConstants={
+      new java.lang.Boolean(true),
+      new java.lang.Byte((byte)1),
+      new java.lang.Character((char)1),
+      new java.lang.Short((short)1),
+      new java.lang.Integer(1),
+      new java.lang.Long(1),
+      new java.lang.Float((float)1.0),
+      new java.lang.Double(1.0)};
+
+    int npbcActual=0;
+
+    for(int i=0;i<8;i++)
+      for(int j=0;j<8;j++) {
+        // determine the result type independently
+        int resID=-1;
+        try {
+          OPlist list=new OPlist();
+          TypesStack typeStk=new TypesStack();
+          java.util.Stack paramOPs=new java.util.Stack();
+          OP o1=new OPload(typeStk,typeConstants[i]);
+          list.addLast(o1);
+          paramOPs.push(o1);
+
+          OP o2=new OPload(typeStk,typeConstants[j]);
+          list.addLast(o2);
+          paramOPs.push(o2);
+
+          list.addLast(new OPbinary(typeStk,paramOPs,code,list));
+          resID=typeStk.peekID();
+          npbcActual++;
+        } catch (CompilationException exc) {
+        };
+        Object res=null;
+        if (resID>=0) { // result exists test it
+          // construct the resulting object
+          switch (resID) {
+          case 0:
+            res=new java.lang.Boolean(resVal>0?true:false);
+            break;
+          case 1:
+            res=new java.lang.Byte((byte)resVal);
+            break;
+          case 2:
+            res=new java.lang.Character((char)resVal);
+            break;
+          case 3:
+            res=new java.lang.Short((short)resVal);
+            break;
+          case 4:
+            res=new java.lang.Integer(resVal);
+            break;
+          case 5:
+            res=new java.lang.Long(resVal);
+            break;
+          case 6:
+            res=new java.lang.Float(resVal);
+            break;
+          case 7:
+            res=new java.lang.Double(resVal);
+            break;
+          default:
+            o.print("*** : ");            
+            o.println("The result of binary operation \""+opSymbols[code]+
+                      "\" on "+TypesStack.specialTypes[i]+" and "+
+                      TypesStack.specialTypes[j]+
+                      " is not primitive type ID="+resID);
+            FAIL(o);
+          };
+        };
+
+        for(int k=0;k<prefixes.length;k++) {
+          for(int m=k;m<prefixes.length;m++) {
+            String op1=prefixes[k]+typeNames[i]+suffixes[k];
+            String expr=op1+
+              opSymbols[code]+prefixes[m]+typeNames[j]+suffixes[m];
+            if (res!=null)
+              testExpression(expr,res,null,context,lib,o);
+            else
+              testError(expr,null,lib,op1.length()+1,o);              
+          };
+        };
+        
+      };
+    
+    o.print("*=*=*= : the total number of successful operations "+npbcActual);
+    if (npbcActual==npbc) {
+      o.println(".");
+      OK(o);
+    } else {
+      o.println(" must be "+npbc);
+      FAIL(o);
+    };
+  };
+
   // Tests evaluation of logical expressions
   // The input is an expression of the form "a&b|c&d"
   // where there are n<=32 free parameters _a,_b,_c,_d (starting from a_ in 
@@ -500,7 +786,6 @@ public class TestSuite {
         if (result!=null) {
           o.print(" ="); o.println(result);
           testok=testok && (result.equals(tobe));
-          if (!result.equals(tobe) && (abort_at_error)) System.exit(1);
         } else {
           o.println("NO RESULT");
           testok=false;
@@ -551,6 +836,7 @@ public class TestSuite {
   private static void FAIL(PrintStream o) {
     failures++;
     o.println(" TEST FAILED !!!"); //o.println(); 
+    if (abort_at_error) System.exit(1);
   };
 
   private static void OK(PrintStream o) {
